# K torch init file

### version 0.0.7 : 14-05-19
# Fixed error in validation set given as input in fit method

### version 0.0.6 : 13-05-19
# Fixed error in predict function. Converted X_test to np.float type


### version 0.0.4 : 10-05-19
# Fixed layer returning 3 params

### version 0.0.3 : 10-05-19
# changed nb_epochs to epochs in models.fit
# changed validation_data from dict to list in models.fit
# added type casting to np.float64 to X and y in models.fit

name = 'k_torch'